// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#ifndef STRING_TOKENIZER_TYPES_H
#define STRING_TOKENIZER_TYPES_H

typedef struct STRING_TOKEN_TAG* STRING_TOKENIZER_HANDLE;

#endif  /*STRING_TOKENIZER_TYPES_H*/
