    utf8_checker_is_valid_utf8
    uws_client_close_async
    uws_client_close_handshake_async
    uws_client_create
    uws_client_create_with_io
    uws_client_destroy
    uws_client_dowork
    uws_client_open_async
    uws_client_retrieve_options
    uws_client_send_frame_async
    uws_client_set_option
    uws_frame_encoder_encode
    wsio_close
    wsio_create
    wsio_destroy
    wsio_dowork
    wsio_get_interface_description
    wsio_open
    wsio_retrieveoptions
    wsio_send
    wsio_setoption
